# -*- coding: iso-8859-15 -*-
# frogdemo.py
#
# frog version 0.87 or higher is needed (also works with python 3.x)
# 5.12.2008 by Marco Haase

from frog import *
from random import random,randrange

# ------ functions for page_0

def drawflagD(x=0,y=0,w=200):
    """Draw German flag at pos (x,y)=center with width w."""
    h = 0.6*w
    d = h/3.0
    flag = Frog(pool,visible=False)
    flag.speed = "max"
    flag.width = d
    flag.pos = x-w/2,y+h/2-0.5*d
    flag.move(w)
    flag.color = "#dd0000" # red
    flag.pos = x-w/2,y+h/2-1.5*d
    flag.move(w)
    flag.color = "#ffce00" # golden
    flag.pos = x-w/2,y+h/2-2.5*d
    flag.move(w)
    flag.color = pool.bgcolor
    flag.pos = x-w/2-0.3*d,y+h/2-3.3*d
    flag.width = 0.6*d
    for k in range(2):
        flag.move(w+0.6*d); flag.turn(90);
        flag.move(h+0.6*d); flag.turn(90)

def drawflagGB(x=0,y=0,w=200):
    """Draw Union Jack at pos (x,y)=center with width w."""
    h = 0.5*w
    jack = Frog(pool,visible=False)
    jack.speed = "max"
    jack.color, jack.fillcolor = "#00247d", "#00247d" # blue
    jack.fill = True
    jack.pos = x-w/2, y-h/2
    for k in range(2):
        jack.move(w); jack.turn(90);
        jack.move(h); jack.turn(90)
    jack.fill = False
    jack.color = "white"
    jack.width = h/5.0
    jack.pos = x-w/2, y-h/2
    jack.moveto(x+w/2,y+h/2)
    jack.pos = x-w/2, y+h/2
    jack.moveto(x+w/2,y-h/2)
    jack.color = "#cf142b" # red
    jack.width = h/15.0
    jack.pos = x-w/2+jack.width,y-h/2
    jack.moveto(x+jack.width,y)
    jack.pos = x-w/2,y+h/2-jack.width/2
    jack.moveto(x,y-jack.width/2)
    jack.pos = x+w/2-jack.width,y+h/2
    jack.moveto(x-jack.width,y)
    jack.pos = x+w/2,y-h/2+jack.width/2
    jack.moveto(x,y+jack.width/2)
    jack.color = "white"
    for k in range(1,3):
        jack.width = h/(4.0*k)
        jack.pos = x-w/2, y
        jack.moveto(x+w/2,y)
        jack.pos = x, y-h/2
        jack.moveto(x,y+h/2)
        jack.color = "#cf142b" # red
    jack.color = pool.bgcolor
    jack.width = h/4.0
    jack.pos = x-w/2-jack.width/2, y-h/2-jack.width/2
    for k in range(2):
        jack.move(w+jack.width); jack.turn(90);
        jack.move(h+jack.width); jack.turn(90)

def inrectangle(q,p1,p2):
    """Check if point q is in rectangle with corners p1 (=upperleft)
    and p2 (=lowerright)."""
    return False if q[0]<p1[0] or q[0]>p2[0] or q[1]<p2[1] or q[1]>p1[1] else True

def chooselanguage(event):
    """Choose language according to flag-click."""
    if pool.cursor != "hand1": return
    global GERMAN
    b1, b2 = (100,110),(300,-10) # D
    q = event["pos"]
    GERMAN = True if inrectangle(q,b1,b2) else False
    pool.cursor = ""
    pool.listen("<Motion>",None)
    page_1()

def changecursor_overflag(event):
    """"change cursor when entering a flag"""
    a1, a2 = (-310,105),(-90,-5) # GB
    b1, b2 = (100,110),(300,-10) # D
    q = event["pos"]
    pool.cursor = "hand1" if inrectangle(q,a1,a2) or inrectangle(q,b1,b2) else ""

# ----- functions for page_3

def gotoclick(gotype,event):
    if pool.cursor == "": return # outside the field
    if event["name"] == "Button-1":
        f = turtle
    elif event["name"] == "Button-2":
        f = arrow
    elif event["name"] == "Button-3":
        f = frog
    else: return
    target = event["pos"]
    f.angle = f.angleto(target)
    if gotype == "jump":
        f.jump(f.distanceto(target))
        f.dot()
    else:
        f.move(f.distanceto(target))
    dist = tuple([f.way/pool.resolution*2.54 for f in frogs])
    if GERMAN:
        text = "Entfernungsmesser  -  Schildkrte: %.1f cm  -  Pfeil: %.1f cm  -  Frosch: %.1f cm" %dist
    else:
        text = "Distance counter  -  turtle: %.1f cm  -  arrow: %.1f cm  -  frog: %.1f cm" %dist
    writer.clear(writer.lastitem)
    writer.write(text)

def jumptoclick(event):
    gotoclick("jump",event)

def movetoclick(event):
    gotoclick("move",event)

def changecursor_inside(event):
    a1, a2 = (-350,150),(350,-250)
    q = event["pos"]
    pool.cursor = "crosshair" if inrectangle(q,a1,a2) else ""

# functions for page_4

def countdown(event):
    global counter
    if ticker.lastitem:
        ticker.clear(ticker.lastitem)
        writer.clear(writer.lastitem)
        counter -= 1
        s = str(counter)
        w,h = writer.textsize(s)
        writer.pos = -w/2, -h/2
        writer.write(s)
        pool.listen("<Timer-1000>",countdown)
    else:
        pool.listen("<Timer>",None)
        writer.clear(writer.lastitem)
        w,h = writer.textsize("bye")
        writer.pos = -w/2, -h/2
        writer.write("Bye")


# -------- pages

def page_0():
    """Choose language"""
    writer.speed = "max"
    writer.color = "forestgreen"
    writer.font = headfont
    text = "Welcome to the frog module!"
    writer.pos = -writer.textsize(text)[0]/2,180
    # insert char by char output
    writer.write(text)
    drawflagGB(-200,50,220)
    drawflagD(200,50,200)
    writer.font = basefont[0],16
    text = "- English version -\nClick at the flag!"
    writer.pos = -200-writer.textsize(text)[0]/2,-80
    writer.write(text)
    text = " - Deutsche Fassung -\nAuf die Flagge klicken!"
    writer.pos = 200-writer.textsize(text)[0]/2,-80
    writer.write(text)
    pool.listen("<Button-1>",chooselanguage)
    pool.listen("<Motion>",changecursor_overflag)


def page_1():
    """First page: Introduce the frogs"""
    if TESTMODE:
        for f in frogs: f.speed = 10
    for f in pool.frogs: f.clear()
    writer.font = headfont
    text = "Darf ich vorstellen:" if GERMAN else "May I introduce:"
    writer.pos = -writer.textsize(text)[0]/2,150
    for ch in text:
        writer.color = random(),random(),random()
        writer.write(ch)
        if not TESTMODE:
            writer.wait(200)
        d = writer.textsize(ch)[0] if ch != " " else writer.textsize("n")[0]
        writer.jump(d)
    # introduce turtle
    turtle.color = "brown"
    turtle.bodycolor = "salmon"
    turtle.shape = "turtle"
    turtle.angle = 90
    turtle.visible = True
    turtle.jumpto(-200,0)
    writer.font = basefont[0],14
    writer.color = turtle.color
    text = "Ich bin die Schildkrte." if GERMAN else "I am the turtle."
    writer.pos = -200-writer.textsize(text)[0]/2,40
    writer.write(text)
    # introduce arrow
    arrow.color = "blue"
    arrow.bodycolor = "thistle"
    arrow.angle = 90
    arrow.visible = True
    writer.color = arrow.color
    text = "Mich nennt man Pfeil." if GERMAN else "I'm called arrow."
    writer.pos = -writer.textsize(text)[0]/2,-40
    writer.write(text)
    # introduce frog
    frog.color = "darkgreen"
    frog.bodycolor = "lightgreen"
    frog.shape = "frog"
    frog.angle = 90
    frog.visible = True
    frog.jumpto(200,0)
    writer.color = frog.color
    text = "Mein Name ist Frosch." if GERMAN else "My name is frog."
    writer.pos = 200-writer.textsize(text)[0]/2,40
    writer.write(text)
    # write info
    writer.font = basefont[0],16
    writer.color = "forestgreen"
    text = "Die Farbe der Figuren lsst sich mit dem Mausrad ndern." if GERMAN \
            else "You can change the shape's color by mousewheel."
    writer.pos = -writer.textsize(text)[0]/2,-150
    writer.write(text)
    writer.memitems = [writer.lastitem]
    writer.font = writer.font[0],20,"bold"
    # writer.font = "Andy MT",24,"bold"
    text = "Zum Fortfahren auf eine der Figuren klicken." if GERMAN \
            else "Please click at one of the shapes to go on."
    writer.pos = -writer.textsize(text)[0]/2,-200
    writer.write(text)
    writer.memitems.append(writer.lastitem)
    # on Mouse over -> rotate!
    turtle.listen("<Enter>",lambda e:pool.setcursor("hand1"),add=True)
    turtle.listen("<Leave>",lambda e:pool.setcursor(""),add=True)
    turtle.listen("<MouseWheel>",lambda e:turtle.setbodycolor((random(),random(),random())),add=True)
    arrow.listen("<Enter>",lambda e:pool.setcursor("hand1"),add=True)
    arrow.listen("<Leave>",lambda e:pool.setcursor(""),add=True)
    arrow.listen("<MouseWheel>",lambda e:arrow.setbodycolor((random(),random(),random())),add=True)
    frog.listen("<Enter>",lambda e:pool.setcursor("hand1"),add=True)
    frog.listen("<Leave>",lambda e:pool.setcursor(""),add=True)
    frog.listen("<MouseWheel>",lambda e:frog.setbodycolor((random(),random(),random())),add=True)
    pool.listen("<Button-1>",page_2)

def page_2(event=None):
    """Second page: move and draw."""
    for f in pool.frogs:
        if f.distanceto(event["pos"])<8:
            f.turn(720)
            pool.listen("<Button-1>",None)
            break
    else: return
    writer.clear()
    turtle.circle(-150,180,draw=False)
    frog.circle(150,180,draw=False)
    phi = 10
    turtle.speed = frog.speed = 10
    for k in range(18):
        turtle.circle(-70,5,draw=False)
        frog.circle(70,5,draw=False)
    arrow.speed = 4
    arrow.y += 30
    arrow.fillcolor = "lightsalmon"
    arrow.fill = True
    for k in range(4):
        arrow.turn(90)
        arrow.circle(80)
        r,g,b = Color.toRGB(arrow.fillcolor)
        r,g,b = min(r+20,255),min(g+20,255),min(b+20,255) # lightup
        arrow.fillcolor = r,g,b
    writer.font = writer.font[0],14
    text = "Klicken um Fortzufahren" if GERMAN else "Press button to continue"
    writer.color = "forestgreen"
    writer.bodycolor = "lightsalmon"
    writer.fillcolor = r,g,b
    writer.pos = -writer.buttonsize(text)[0]/2,-200
    writer.button(text,page_3)

def page_3(event=None):
    """Third page: Let the user draw."""
    writer.clear()
    for k in range(4):
        arrow.clear(arrow.lastitem)
        arrow.wait(800)
    arrow.turn(90)
    arrow.circle(90,180,draw=False)
    arrow.turn(90)
    arrow.jump(40)
    arrow.beep()
    writer.font = writer.font[0],14
    if GERMAN:
        text = "Die Figuren folgen der Maus - klicke mit einer der drei Maustasten.\n"+\
               "Wenn zustzlich die <Strg>-Taste gedrckt ist, wird gezeichnet.\n"+\
               "Geschlossene Figuren werden mit zuflligen Farben ausgefllt.\n"+\
               "Zum Beenden die <Esc>-Taste drcken."
    else:
        text = "The objects follow the mouse - try all three mouse buttons.\n"+\
               "Hold the <Ctrl>-key while pressing mouse button to draw.\n"+\
               "Closed shapes will be filled with random colors.\n"+\
               "Press <Esc> to exit."
    writer.pos = -350,-250
    for k in range(2):
        writer.move(700)
        writer.turn(90)
        writer.move(400)
        writer.turn(90)
    writer.pos = -writer.textsize(text)[0]/2, 200
    writer.write(text)
    writer.memitems = writer.lastitem
    # prepare frogs to fill
    turtle.fill = frog.fill = arrow.fill = True
    randomcolor = lambda : (random(),random(),random())
    turtle.fillcolor = randomcolor()
    arrow.fillcolor = randomcolor()
    frog.fillcolor = randomcolor()
    # create statusbar and distance-counter
    writer.color = "ivory"
    writer.fillcolor = "ivory"
    writer.fill = True
    writer.pos = 400,-300
    writer.angle = 90
    for k in range(2):
        writer.move(30)
        writer.turn(90)
        writer.move(800)
        writer.turn(90)
    writer.font = basefont
    writer.color = "saddlebrown"
    writer.pos = -350,-292
    if GERMAN:
        text = "Entfernungsmesser  -  Schildkrte: 0.0 cm  -  Pfeil: 0.0 cm  -  Frosch: 0.0 cm"
    else:
        text = "Distance counter  -  turtle: 0.0 cm  -  arrow: 0.0 cm  -  frog: 0.0 cm"
    writer.write(text)
    for f in frogs:
        f.way = 0
    pool.listen("<Button>",jumptoclick)
    pool.listen("<Control-Button>",movetoclick)
    pool.listen("<Motion>",changecursor_inside)
    pool.listen("<Key-Escape>",page_4)

def page_4(event=None):
    """Last page: Say goodbye."""
    pool.cursor = ""
    for f in pool.frogs:
        f.clear()
        f.visible = False
    pool.listen("<Button>",None)
    pool.listen("<Control-Button>",None)
    pool.listen("<Motion>",None)
    pool.listen("<Key-Escape>",None)
    # draw countdown
    global counter
    counter = 12
    writer.font = headfont
    writer.color = "forestgreen"
    ticker.speed = "max"
    ticker.color = "orange"
    ticker.width = 5
    ticker.angle = 90
    for k in range(12):
        ticker.jump(60)
        ticker.move(10)
        ticker.jump(-70)
        ticker.turn(-30)
    s = str(counter)
    w,h = writer.textsize(s)
    writer.pos = -w/2, -h/2
    writer.write(s)
    pool.listen("<Timer-1000>",countdown)


# START OF MAIN PROGRAM -------------------------------------

TESTMODE = False # increase speed while testing
GERMAN = True # language - might be changed by user

# create Pool and Frogs
pool = Pool(width=800,height=600,bgcolor="linen",title="The Pool - Frog Demo")
arrow = Frog(pool,visible=False)
turtle = Frog(pool,visible=False)
frog = Frog(pool,visible=False)
frogs = [turtle,arrow,frog]
writer = Frog(pool,visible=False)
ticker = Frog(pool,visible=False)
basefont = writer.font
headfont = ("times",32,"bold") if platform.system() == "Linux" else ("Times",32,"bold")
page_0()
pool.ready()
